package ru.dbotthepony.mc.otm.client.screen.panels

import net.minecraft.network.chat.Component
import ru.dbotthepony.kommons.math.RGBAColor
import ru.dbotthepony.mc.otm.client.render.GraphMouseLabels
import ru.dbotthepony.mc.otm.client.render.MGUIGraphics
import ru.dbotthepony.mc.otm.client.render.renderGraph
import ru.dbotthepony.mc.otm.client.screen.MatteryScreen
import ru.dbotthepony.mc.otm.core.DecimalHistoryGraph
import ru.dbotthepony.mc.otm.core.TextComponent
import ru.dbotthepony.mc.otm.core.math.Decimal

open class DecimalHistoryGraphPanel<out S : MatteryScreen<*>>(
	screen: S,
	parent: EditablePanel<*>,
	val graph: DecimalHistoryGraph,
	val formatText: (Decimal) -> Component = { TextComponent(it.toString(2)) },
	x: Float = 0f,
	y: Float = 0f,
	width: Float = 10f,
	height: Float = 10f
) : EditablePanel<S>(screen, parent, x, y, width, height) {
	override fun innerRender(graphics: MGUIGraphics, mouseX: Float, mouseY: Float, partialTick: Float) {
		val maximum = graph.max()
		val normalized: FloatArray

		if (maximum.isZero || maximum.isInfinite) {
			normalized = FloatArray(graph.width) {
				if (graph[it].isInfinite) 0.8f else 0.0f
			}
		} else {
			normalized = FloatArray(graph.width) { (graph[it] / maximum).toFloat() * 0.9f }
		}

		graphics.renderRect(0f, 0f, this.width, this.height, color = RGBAColor.BLACK)

		renderGraph(
			graphics.pose,
			normalized,
			width,
			height,
			labels = GraphMouseLabels(
				mouseX - absoluteX,
				mouseY - absoluteY,
				{ formatText(graph[it]) },
				font,
			)
		)
	}

	companion object {

	}
}
